from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

__version__ = '2.1.0'

# Retrieved from Chrome document.featurePolicy.allowedFeatures()
FEATURE_NAMES = [
    'accelerometer',
    'ambient-light-sensor',
    'autoplay',
    'camera',
    'document-domain',
    'document-write',
    'encrypted-media',
    'font-display-late-swap',
    'fullscreen',
    'geolocation',
    'gyroscope',
    'layout-animations',
    'lazyload',
    'legacy-image-formats',
    'magnetometer',
    'microphone',
    'midi',
    'oversized-images',
    'payment',
    'picture-in-picture',
    'speaker',
    'sync-script',
    'sync-xhr',
    'unoptimized-images',
    'unsized-media',
    'usb',
    'vertical-scroll',
    'vr',
    'wake-lock',
]


class FeaturePolicyMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        self.get_header_value()  # to check

    def __call__(self, request):
        response = self.get_response(request)
        value = self.get_header_value()
        if value:
            response['Feature-Policy'] = value
        return response

    def get_header_value(self):
        setting = getattr(settings, 'FEATURE_POLICY', {})
        pieces = []
        for feature, values in setting.items():
            if feature not in FEATURE_NAMES:
                raise ImproperlyConfigured('Unknown feature {}'.format(feature))
            if isinstance(values, str):
                values = (values,)

            item = [feature]
            for value in values:
                if value == '*':
                    item.append("'*'")
                elif value == 'none':
                    item.append("'none'")
                elif value == 'self':
                    item.append("'self'")
                else:
                    item.append(value)
            pieces.append(' '.join(item))
        return '; '.join(pieces)



DEBUG = False

SECRET_KEY = 'not-secret'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'LOCATION': ':memory:',
    },
}

ALLOWED_HOSTS = ['*']

INSTALLED_APPS = []

MIDDLEWARE = [
    'django_feature_policy.FeaturePolicyMiddleware',
    'django.middleware.common.CommonMiddleware',
]

ROOT_URLCONF = 'tests.testapp.urls'

STATIC_URL = '/static/'

TEMPLATES = []



import pytest
from django.core.exceptions import ImproperlyConfigured


def test_index(client):
    resp = client.get('/')

    assert resp.status_code == 200
    assert resp.content == b'Hello World'


def test_no_setting(client):
    resp = client.get('/')

    assert 'Feature-Policy' not in resp


def test_anyone_can_geolocate(client, settings):
    settings.FEATURE_POLICY = {'geolocation': '*'}

    resp = client.get('/')

    assert resp['Feature-Policy'] == "geolocation '*'"


def test_anyone_can_geolocate_list(client, settings):
    settings.FEATURE_POLICY = {'geolocation': ['*']}

    resp = client.get('/')

    assert resp['Feature-Policy'] == "geolocation '*'"


def test_no_one_can_geolocate(client, settings):
    settings.FEATURE_POLICY = {'geolocation': 'none'}

    resp = client.get('/')

    assert resp['Feature-Policy'] == "geolocation 'none'"


def test_self_can_geolocate(client, settings):
    settings.FEATURE_POLICY = {'geolocation': 'self'}

    resp = client.get('/')

    assert resp['Feature-Policy'] == "geolocation 'self'"


def test_example_com_can_geolocate(client, settings):
    settings.FEATURE_POLICY = {'geolocation': 'https://example.com'}

    resp = client.get('/')

    assert resp['Feature-Policy'] == 'geolocation https://example.com'


def test_multiple_allowed(client, settings):
    settings.FEATURE_POLICY = {
        'autoplay': ['self', 'https://example.com'],
    }

    resp = client.get('/')

    assert resp['Feature-Policy'] == "autoplay 'self' https://example.com"


def test_multiple_features(client, settings):
    settings.FEATURE_POLICY = {
        'accelerometer': 'self',
        'geolocation': ['self', 'https://example.com'],
    }

    resp = client.get('/')

    assert resp['Feature-Policy'] == "accelerometer 'self'; geolocation 'self' https://example.com"


def test_unknown_feature(client, settings):
    settings.FEATURE_POLICY = {'accelerometor': 'self'}

    with pytest.raises(ImproperlyConfigured):
        client.get('/')
